/**
 * Use a single state. Returns a tuple with `[state, update, reset, replace]`. Set `label` to log a diff of state changes, and force logging on or off with `enableLog`.
 *
 * Basic usage:
 *
 * 	const [state, update, reset, replace] = useSingleState({note: '', value: 0} [, label, enableLog]);
 *
 * - `state` is your current state object
 * - `update(object)` takes an update object, and creates your next state by merging it with your current state
 * - `reset()` resets your state to its initial value
 * - `replace(object)` takes a replacement object, which becomes your next state
 *
 * Examples:
 *
```typescript
const { note, value } = state;
update({
    note: 'New note'
});
// Update is now { text: '', value: 0, note: '' }

reset();
// State is now { text: '', value: 0, note: '' }

replace({
    value: 4
})
```
 * @param initialState Initialises the state
 * @param label (Optional) A label for the component
 * @param enableLog (Optional) Enable logging. Defaults to true.
 */
declare const useSingleState: <T extends object>(initialState: T, label?: string, enableLog?: boolean) => [T, (updateState: Partial<T>) => void, () => void, (newState: T) => void];
export default useSingleState;
