"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = require("react");
var diffLogger_1 = __importDefault(require("./utils/diffLogger"));
/**
 * Use a single state. Returns a tuple with `[state, update, reset, replace]`. Set `label` to log a diff of state changes, and force logging on or off with `enableLog`.
 *
 * Basic usage:
 *
 * 	const [state, update, reset, replace] = useSingleState({note: '', value: 0} [, label, enableLog]);
 *
 * - `state` is your current state object
 * - `update(object)` takes an update object, and creates your next state by merging it with your current state
 * - `reset()` resets your state to its initial value
 * - `replace(object)` takes a replacement object, which becomes your next state
 *
 * Examples:
 *
```typescript
const { note, value } = state;
update({
    note: 'New note'
});
// Update is now { text: '', value: 0, note: '' }

reset();
// State is now { text: '', value: 0, note: '' }

replace({
    value: 4
})
```
 * @param initialState Initialises the state
 * @param label (Optional) A label for the component
 * @param enableLog (Optional) Enable logging. Defaults to true.
 */
var useSingleState = function (initialState, label, enableLog) {
    if (label === void 0) { label = null; }
    if (enableLog === void 0) { enableLog = true; }
    var _a = react_1.useState(initialState), state = _a[0], set = _a[1];
    var replace = function (updateState) {
        if (label && enableLog) {
            diffLogger_1.default(state, updateState, label);
        }
        set(updateState);
    };
    var update = function (updateState) {
        return replace(__assign({}, state, updateState));
    };
    var reset = function () { return replace(initialState); };
    return [state, update, reset, replace];
};
exports.default = useSingleState;
